﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class HUDManager : MonoBehaviour
{

    static private HUDManager _instance;

    static public HUDManager GetInstance()
    {
        if (_instance == null)
        {
            _instance = new HUDManager();
        }

        return _instance;
    }

    HUDManager()
    {
        if (_instance == null)
        {
            _instance = this;
        }
    }


    private Text candyText = null;
    private Text candyStashText = null;
    private Text debugText = null;
    private Text jackpotText = null;
    private Text clockText = null;

    // Use this for initialization
    void Start()
    {
        candyText = gameObject.transform.Find("CandyText").gameObject.GetComponent<Text>();
        candyStashText = gameObject.transform.Find("CandyStashText").gameObject.GetComponent<Text>();
        debugText = gameObject.transform.Find("DebugText").gameObject.GetComponent<Text>();
        clockText = gameObject.transform.Find("Clock").gameObject.GetComponent<Text>();

        homeArrowImage = gameObject.transform.Find("HomeArrow").gameObject.GetComponent<Image>();
        homeArrowOuterImage = homeArrowImage.transform.Find("HomeArrowOuter").gameObject.GetComponent<Image>();

        jackpotText = gameObject.transform.Find("JackpotText").gameObject.GetComponent<Text>();

        homeObj = GameObject.FindGameObjectWithTag("Home");
        playerObj = GameObject.FindGameObjectWithTag("Player");

        EventManager.instance.AddListener<CandyChangedEvent>(OnCandyChanged);

        UpdateCandyDisplays();

    }

    public void UpdateCandyDisplays()
    {
        PlayerStats ps = PlayerStats.GetInstance();
        SetCandyCount(ps.Candy, ps.CandyCapacity);
        SetCandyStashCount(ps.CandyStash);

    }

    public void OnDestroy()
    {
        EventManager.instance.RemoveListener<CandyChangedEvent>(OnCandyChanged);
    }

    public void OnCandyChanged(CandyChangedEvent e)
    {
        UpdateCandyDisplays();
    }

    void UpdateTime()
    {
        int Minutes = Mathf.FloorToInt(Time.timeSinceLevelLoad % 60.0f);
        int Hours = Mathf.FloorToInt(Time.timeSinceLevelLoad / 60.0f) + 6;

        string s = string.Format("{0}:{1:D2}PM", Hours, Minutes);
        clockText.text = s;
    }

    public void SetClockColor(Color c)
    {
        clockText.color = c;
    }

    // Update is called once per frame
    void Update()
    {
        UpdateHomeArrow();
        UpdateTime();
    }

    public void ShowJackpotText()
    {
        jackpotText.gameObject.SetActive(true);
        Invoke("HideJackpotText", 2);
    }

    public void HideJackpotText()
    {
        jackpotText.gameObject.SetActive(false);
    }

    private Image homeArrowImage = null;
    private Image homeArrowOuterImage = null;

    private GameObject homeObj = null;
    private GameObject playerObj = null;

    void UpdateHomeArrow()
    {
        Vector3 dirHome = homeObj.transform.position - playerObj.transform.position;

        if (dirHome.sqrMagnitude > 225.0f)
        {
            // Show the arrow if you're far
            homeArrowImage.gameObject.SetActive(true);

            // Reposition the arrow
            RectTransform canvasBounds = gameObject.GetComponent<RectTransform>();
            float angleToHome = Mathf.Atan2(dirHome.y, dirHome.x);

            float xDistance = (canvasBounds.rect.width - 60) / 2f;
            float yDistance = (canvasBounds.rect.height - 60) / 2f;

            float xPos = xDistance * Mathf.Cos(angleToHome);
            float yPos = yDistance * Mathf.Sin(angleToHome);

            homeArrowImage.gameObject.transform.localPosition = new Vector3(xPos, yPos, homeArrowImage.gameObject.transform.localPosition.z);

            // Rotate the arrow
            homeArrowOuterImage.rectTransform.rotation = Quaternion.Euler(0.0f, 0.0f, Mathf.Rad2Deg * angleToHome - 90f);
        }
        else
        {
            // Hide the arrow if you're close
            homeArrowImage.gameObject.SetActive(false);
        }


    }

    public void SetDebugString(string s)
    {
        if (debugText != null)
        {
            debugText.text = s;
        }
    }

    public void SetCandyCount(int CandyCount, int Capacity)
    {
        if (candyText != null)
        {
            candyText.text = string.Format("Candy: {0} / {1}", CandyCount, Capacity);
        }
    }

    public void SetCandyStashCount(int StashCount)
    {
        if (candyStashText != null)
        {
            candyStashText.text = "Stored Candy: " + StashCount;
        }
    }

}
