﻿using UnityEngine;
using System.Collections;

public class PlayerController : MonoBehaviour
{

    public float DefaultMovementSpeed = 6.0f;
    public float SidewalkSpeedMultiplier = 2.0f;

    private float TargetSpeed = 6.0f;
    private float CurrentSpeed = 6.0f;

    private Rigidbody2D myRigidBody = null;
    Animator animator = null;
    // Use this for initialization
    void Start()
    {
        myRigidBody = gameObject.GetComponent<Rigidbody2D>();
        animator = gameObject.GetComponent<Animator>();
    }

    private const float DeAccelerationCap = -0.15f;
    private const float AccelerationCap = 0.50f;

    public void FixedUpdate()
    {
        float slowCap = DeAccelerationCap;
        if (PlayerStats.GetInstance().HasUpgrade(CandyUpgrade.FastSidewalk))
            slowCap *= 2.0f;

        CurrentSpeed += Mathf.Clamp(TargetSpeed - CurrentSpeed, slowCap, AccelerationCap);
    }

    public enum MoveDirection
    {
        None,
        Up,
        Down,
        Left,
        Right
    }

    MoveDirection lastDir = MoveDirection.None;

    // Update is called once per frame
    void Update()
    {
        TargetSpeed = DefaultMovementSpeed;

        // Check to see if I'm overlapping sidewalk
        bool isTouchingSidewalk = Physics2D.IsTouchingLayers(GetComponent<Collider2D>(), LayerMask.GetMask("Sidewalk"));

        if (isTouchingSidewalk)
        {
            TargetSpeed *= SidewalkSpeedMultiplier;
            if (PlayerStats.GetInstance().HasUpgrade(CandyUpgrade.FastSidewalk))
            TargetSpeed *= 1.25f;
        }
        else if (PlayerStats.GetInstance().HasUpgrade(CandyUpgrade.FastGrass))
        {
            TargetSpeed *= 1.5f;
        }

        float x = Input.GetAxis("Horizontal");
        float y = Input.GetAxis("Vertical");

        Vector3 dir = new Vector3(x, y, 0f);

        MoveDirection newDir = MoveDirection.None;

        // Turn the player based on input
        if (dir.sqrMagnitude > Mathf.Epsilon)
        {
            //// Move Assist
            //Vector3 Forward = Vector3.up;

            //// Pick a facing
            //if (Mathf.Abs(dir.x) - Mathf.Abs(dir.y) > 0.50f)
            //{
            //    // face along the x-axis
            //    Forward = new Vector3(Mathf.Sign(dir.x), 0, 0);
            //}
            //else if (Mathf.Abs(dir.y) - Mathf.Abs(dir.x) > 0.50f)
            //{
            //    // face along the y-axis
            //    Forward = new Vector3(0, Mathf.Sign(dir.y), 0);
            //}

            //// dir = MoveAssist(gameObject.GetComponent<CircleCollider2D>().transform.position, Forward, dir);

                // Start Moving
            animator.SetBool("Stopped", false);

            if (Mathf.Abs(x) > Mathf.Abs(y))
            {
                // Point left or right
                if (x > 0.0f)
                    newDir = MoveDirection.Right;
                else
                    newDir = MoveDirection.Left;
            }
            else
            {
                // Point up or down
                if (y > 0.0f)
                    newDir = MoveDirection.Up;
                else
                    newDir = MoveDirection.Down;
            }

        }
        else
        {
            // Stop moving
            animator.SetBool("Stopped", true);
        }

        if (newDir != lastDir)
        {
            lastDir = newDir;
            switch(newDir)
            {
                case MoveDirection.Right:
                    animator.SetTrigger("Right");
                    break;
                case MoveDirection.Left:
                    animator.SetTrigger("Left");
                    break;
                case MoveDirection.Up:
                    animator.SetTrigger("Up");
                    break;
                case MoveDirection.Down:
                    animator.SetTrigger("Down");
                    break;
                default:
                    break;
            }
        }


        if (myRigidBody != null)
        {
            myRigidBody.velocity = dir * CurrentSpeed;
        }
        else
        {
            print("Rigidbody on player missing this is bad.");
        }

    }


    private Vector2 MoveAssist(Vector3 origin, Vector3 forward, Vector3 dir3D)
    {
        const float probeAngle = 44.0f;

        // first check if we need move assist. are we running into anything?
        int layerMask = LayerMask.GetMask("Default");

        // const float probeLength = 0.64f;
        const float probeLength = 5.0f;
        // ok, there is something blocking us. make a pair of probe vectors
        Vector3 leftProbe = Quaternion.AngleAxis(-probeAngle, Vector3.back) * forward;
        Vector3 rightProbe = Quaternion.AngleAxis(probeAngle, Vector3.back) * forward;


        // check if there is a clear area to the left or right.
        bool bLeftBlocked = Physics.Raycast(origin, leftProbe, probeLength, layerMask);
        bool bRightBlocked = Physics.Raycast(origin, rightProbe, probeLength, layerMask);

        // HUDManager.GetInstance().SetDebugString("Left Blocked: " + bLeftBlocked + "\nRight Blocked: " + bRightBlocked);

        if (!bLeftBlocked && bRightBlocked)
        {
            // rotate to the left
            dir3D = Quaternion.Euler(0.0f, 0.0f, 60.0f) * dir3D;
        }
        else if (!bRightBlocked && bLeftBlocked)
        {
            // rotate to the right
            dir3D = Quaternion.Euler(0.0f, 0.0f, -60.0f) * dir3D;
        }

        var result = new Vector2(dir3D.x, dir3D.y);
        result.Normalize();

        return result;
    }

}


