﻿using UnityEngine;
using System.Collections;

public class CandyChangedEvent : GameEvent
{
}

public enum CandyUpgrade
{
    First = 0,
    FastGrass = First,
    FastSidewalk,
    CarryingCapacity,
    BonusCandyFromHouses,
    Flashlight,
    Firecrackers,
    NumUpgrades
}


public class PlayerStats {

    // ----------------------------------------------------------------------------
    // Upgrades
    //

    public bool[] Upgrades = new bool[(int)CandyUpgrade.NumUpgrades];
    public int[] UpgradeCosts =
    {
        60,  // FastGrass
        140,  // Fast Sidewalk
        100, // Carrying Capacity
        80,  // Bonus Candy From Houses
        180, // Flashlight
        210, // Firecrackers
    };

    private static PlayerStats _instance = null;

    public static PlayerStats GetInstance()
    {
        if (_instance == null)
            _instance = new PlayerStats();

        return _instance;

    }

    public PlayerStats()
    {
        for (CandyUpgrade i = CandyUpgrade.FastGrass; i < CandyUpgrade.NumUpgrades; i++)
        {
            Upgrades[(int)i] = false;
        }
    }

    public void StartNewRound()
    {
        _candy = 0;
        _candyStash = 0;
    }

    private int _candy = 0;
    public int Candy
    {
        get { return _candy; }
        set
        {
            _candy = value;
            if (_candy > CandyCapacity)
            {
                _candy = CandyCapacity;
            }

            EventManager.instance.Raise(new CandyChangedEvent());
        }
    }

    private int _candyStash = 0;
    public int CandyStash
    {
        get { return _candyStash; }
        set {
            _candyStash = value;
            EventManager.instance.Raise(new CandyChangedEvent());
        }
    }

    public void DropOffCandy()
    {
        _candyStash += _candy;
        _candy = 0;
        EventManager.instance.Raise(new CandyChangedEvent());

    }

    public void GiveCandy(int candies)
    {
        Candy += candies;
    }

    public bool HasUpgrade(CandyUpgrade u)
    {
        return Upgrades[(int)u];
    }

    public int GetUpgradeCost(CandyUpgrade u)
    {
        if (u < CandyUpgrade.NumUpgrades)
        {
            return UpgradeCosts[(int)u];
        }

        return 0;
    }

    public void EnableUpgrade(CandyUpgrade u)
    {
        Upgrades[(int)u] = true;
    }

    public bool TryToPurchaseUpgrade(CandyUpgrade u)
    {
        bool success = false;
        // Check for enough candy
        if (_candyStash >= GetUpgradeCost(u))
        {
            success = true;

            _candyStash -= GetUpgradeCost(u);
            EnableUpgrade(u);
        }

        return success;
    }

    public int CandyCapacity
    {
        get
        {
            if (HasUpgrade(CandyUpgrade.CarryingCapacity))
                return 80;

            return 40;
        }
    }

}
