﻿using UnityEngine;
using UnityEngine.UI;
using System.Linq;
using System.Collections;
using System.Collections.Generic;

public class UpgradeScreenHandler : MonoBehaviour {

    private Dictionary<CandyUpgrade, GameObject> upgradeButtonObjs = new Dictionary<CandyUpgrade, GameObject>();

    void IdentifyComponents()
    {
        GameObject uiCanvas = GameObject.Find("Canvas");

        for (CandyUpgrade cu = CandyUpgrade.First; cu < CandyUpgrade.NumUpgrades; cu++)
        {
            string buttonName = "Upgrade" + (int)cu;
            Transform t = uiCanvas.transform.Find(buttonName);
            GameObject go = (t != null) ? t.gameObject : null;

            upgradeButtonObjs.Add(cu, go);
        }
    }

    void UpdateAllPrices()
    {
        for (CandyUpgrade cu = CandyUpgrade.First; cu < CandyUpgrade.NumUpgrades; cu++)
        {
            GameObject o = upgradeButtonObjs[cu];
            GameObject costObj = o.transform.Find("Cost").gameObject;
            costObj.GetComponent<Text>().text = string.Format("{0} Candy", PlayerStats.GetInstance().GetUpgradeCost(cu));
        }
    }

    void RefreshAllButtonStates()
    {
        for (CandyUpgrade cu = CandyUpgrade.First; cu < CandyUpgrade.NumUpgrades; cu++)
        {
            GameObject o = upgradeButtonObjs[cu];
            Button button = o.GetComponent<Button>();
            button.interactable = ! PlayerStats.GetInstance().HasUpgrade(cu);
        }
    }

    // Use this for initialization
    void Start () {
        IdentifyComponents();
        UpdateAllPrices();
        RefreshAllButtonStates();
	}
	
	// Update is called once per frame
	void Update () {

        GameObject uiCanvas = GameObject.Find("Canvas");

        GameObject candyTextObj = uiCanvas.transform.Find("Candy").gameObject;
        candyTextObj.GetComponent<Text>().text = string.Format("{0}", PlayerStats.GetInstance().CandyStash);


        if (Debug.isDebugBuild)
        {
            if (Input.GetKeyDown(KeyCode.M))
            {
                PlayerStats.GetInstance().CandyStash += 10;
            }
        }
    }

    public void OnPlayButton()
    {
        Application.LoadLevel("gameScene");
    }


    public void OnPurchaseUpgradeButton(int upgradeIndex)
    {
        CandyUpgrade cu = (CandyUpgrade)upgradeIndex;

        PlayerStats ps = PlayerStats.GetInstance();
        if (ps.TryToPurchaseUpgrade(cu))
        { 
           RefreshAllButtonStates();
        }
    }
}
